/* Software-based Trusted Platform Module (TPM) Emulator
 * Emulator Copyright (C) 2004-2010 Mario Strasser <mast@gmx.net>
 * Timing/plugin code Copyright (C) 2010-2012 Stephen R. Tate <srtate@uncg.edu>
 *
 * This module is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This module is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#ifndef _PLUGIN_HASHTREE
#define _PLUGIN_HASHTREE

/*
 * "Modes" (or commands) for the TPM_ExecuteHashTree command
 */

#define TPM_HASHTREE_MODE_CREATE 1
#define TPM_HASHTREE_MODE_REUSE 2
#define TPM_HASHTREE_MODE_INCREMENT 3
#define TPM_HASHTREE_MODE_RESET 4
#define TPM_HASHTREE_MODE_READ 5
#define TPM_HASHTREE_MODE_READROOT 6

/*
 * Counter blob - note that counterIDrand is randomized and set by the
 * TPM when the counter is created.  This acts as a nonce so that a
 * user cannot reset and recreate a counter in order to roll it back
 * (if they reset and recreate a counter in this position, the TPM
 * will assign a new random part of the ID, which is different from
 * previous values with overwhelming probability).
 */

typedef struct tdTPM_COUNTER_BLOB {
  UINT32 counterIDpos;
  TPM_NONCE counterIDrand;
  UINT32 countValue;
  TPM_NONCE data;
  UINT32 authDataBlob;
} TPM_COUNTER_BLOB;


/*
 * The data stored inside the TPM (or with the simulator, stored in a
 * file so that it can be restored when the simulator is stopped and
 * restarted).
 */

typedef struct tdTPM_HASHTREE_PERMDATA {
  UINT32 height;
  TPM_DIGEST rootDigest;
} TPM_HASHTREE_PERMDATA;

#define sizeof_TPM_HASHTREE_PERMDATA(s) (4+20)

/*
 * The info returned from the TPM_ExecuteHashTree command - the first
 * three fields are signed by an AIK, and this signature is in the
 * final fields of this return structure.
 */

typedef struct tdTPM_HASHTREE_EXEC_CERT {
    BYTE mode; // 1
    TPM_NONCE nonce; // 20
    TPM_COUNTER_BLOB newCounterBlob; // 52
    UINT32 signatureSize; // 4
    BYTE* signature; // 0 
} TPM_HASHTREE_EXEC_CERT;

#endif


